@@ -19,15 +19,19 @@
  */
 
 #include "box_data.h"
-#include "../data_structure/glomconversions.h"
-#include "../utils.h"
-#include "../data_structure/layout/layoutitem_field.h"
+#include <glom/libglom/data_structure/glomconversions.h>
+#include <glom/libglom/utils.h>
+#include <glom/libglom/data_structure/layout/layoutitem_field.h>
+#include <glom/glom_privs.h>
 #include "../python_embed/glom_python.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <algorithm> //For std::find()
 #include "config.h"
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Box_Data::Box_Data()
 : m_Button_Find(Gtk::Stock::FIND),
   m_pDialogLayout(0)
@@ -47,26 +51,26 @@ Box_Data::~Box_Data()
   }
 }
 
-bool Box_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
+bool Box_Data::init_db_details(const FoundSet& found_set)
 {
-  m_strTableName = strTableName;
-  m_strWhereClause = strWhereClause;
+  m_table_name = found_set.m_table_name;
+  m_found_set = found_set;
 
   create_layout(); //So that fill_from_database() can succeed.
 
-  return Box_DB_Table::init_db_details(strTableName); //Calls fill_from_database().
+  return Box_DB_Table::init_db_details(m_table_name); //Calls fill_from_database().
 }
 
-bool Box_Data::refresh_data_from_database_with_where_clause(const Glib::ustring& strWhereClause)
+bool Box_Data::refresh_data_from_database_with_where_clause(const FoundSet& found_set)
 {
-  m_strWhereClause = strWhereClause;
+  m_found_set = found_set;
 
   return Box_DB_Table::refresh_data_from_database(); //Calls fill_from_database().
 }
 
-Glib::ustring Box_Data::get_where_clause() const
+FoundSet Box_Data::get_found_set() const
 {
-  return m_strWhereClause;
+  return m_found_set;
 }
 
 Glib::ustring Box_Data::get_find_where_clause() const
@@ -80,25 +84,32 @@ Glib::ustring Box_Data::get_find_where_clause() const
 
     const Gnome::Gda::Value data = get_entered_field_data(*iter);
 
-    if(!GlomConversions::value_is_empty(data))
+    if(!Conversions::value_is_empty(data))
     {
       const sharedptr<const Field> field = (*iter)->get_full_field_details();
-
-      bool use_this_field = true;
-      if(field->get_glom_type() == Field::TYPE_BOOLEAN) //TODO: We need an intermediate state for boolean fields, so that they can be ignored in searches.
+      if(field)
       {
-        if(!data.get_bool())
-          use_this_field = false;
-      }
+        bool use_this_field = true;
+        if(field->get_glom_type() == Field::TYPE_BOOLEAN) //TODO: We need an intermediate state for boolean fields, so that they can be ignored in searches.
+        {
+          if(!data.get_bool())
+            use_this_field = false;
+        }
 
-      if(use_this_field)
-      {
-        strClausePart = m_strTableName + "." + field->get_name() + " " + field->sql_find_operator() + " " +  field->sql_find(data); //% is mysql wildcard for 0 or more characters.
+        if(use_this_field)
+        {
+          strClausePart = "\"" + m_table_name + "\".\"" + field->get_name() + "\" " + field->sql_find_operator() + " " +  field->sql_find(data); //% is mysql wildcard for 0 or more characters.
+        }
       }
     }
 
     if(!strClausePart.empty())
-      strClause += strClausePart + " ";
+    {
+      if(!strClause.empty())
+        strClause += "AND ";
+
+      strClause += "(" + strClausePart + ") ";
+    }
   }
 
   return strClause;
@@ -121,52 +132,18 @@ void Box_Data::on_Button_Find()
     signal_find_criteria.emit(where_clause);
 }
 
-Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Value& primary_key_value)
-{
-  type_map_fields field_values;
-
-  sharedptr<const Field> fieldPrimaryKey = get_field_primary_key();
-
-  Document_Glom* document = get_document();
-  if(document)
-  {
-    //TODO: Cache the list of all fields, as well as caching (m_Fields) the list of all visible fields:
-    const Document_Glom::type_vecFields fields = document->get_table_fields(m_strTableName);
-
-    //TODO: This seems silly. We should just have a build_sql_select() that can take this container:
-    type_vecLayoutFields fieldsToGet;
-    for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
-    {
-      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-      layout_item->set_full_field_details(*iter);
-
-      fieldsToGet.push_back(layout_item);
-    }
-
-    const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, fieldPrimaryKey, primary_key_value);
-    Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(query);
-
-    if(data_model && data_model->get_n_rows())
-    {
-      int col_index = 0;
-      for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
-      {
-        //There should be only 1 row. Well, there could be more but we will ignore them.
-        field_values[(*iter)->get_name()] = data_model->get_value_at(col_index, 0);
-        ++col_index;
-      }
-    }
-    else
-    {
-      handle_error();
-    }
-  }
-
-  return field_values;
-}
-
 Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value)
 {
+  return record_new(use_entered_data, primary_key_value, Gtk::TreeModel::iterator());
+}
+
+void Box_Data::set_primary_key_value(const Gtk::TreeModel::iterator& /* row */, const Gnome::Gda::Value& /* value */)
+{
+  //Box_Data_List overrides this.
+}
+
+Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value, const Gtk::TreeModel::iterator& row)
+{  
   sharedptr<const Field> fieldPrimaryKey = get_field_primary_key();
 
   const Glib::ustring primary_key_name = fieldPrimaryKey->get_name();
@@ -188,6 +165,8 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
     }
   }
 
+  Document_Glom* document = get_document();
+
   //Calculate any necessary field values and enter them:
   for(type_vecLayoutFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
   {
@@ -196,30 +175,34 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
     //If the user did not enter something in this field:
     Gnome::Gda::Value value = get_entered_field_data(layout_item);
 
-    if(GlomConversions::value_is_empty(value)) //This deals with empty strings too.
+    if(Conversions::value_is_empty(value)) //This deals with empty strings too.
     {
       const sharedptr<const Field>& field = layout_item->get_full_field_details();
-
-      //If the default value should be calculated, then calculate it:
-      if(field->get_has_calculation())
+      if(field)
       {
-        const Glib::ustring calculation = field->get_calculation();
-        const type_map_fields field_values = get_record_field_values(primary_key_value);
-
-        const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field->get_glom_type(), calculation, field_values,
-          get_document(), m_strTableName);
-        set_entered_field_data(layout_item, value);
-      }
-
-      //Use default values (These are also specified in postgres as part of the field definition,
-      //so we could theoretically just not specify it here.
-      //TODO_Performance: Add has_default_value()?
-      if(GlomConversions::value_is_empty(value))
-      {
-        const Gnome::Gda::Value default_value = field->get_default_value();
-        if(!GlomConversions::value_is_empty(default_value))
+        //If the default value should be calculated, then calculate it:
+        if(field->get_has_calculation())
         {
-          set_entered_field_data(layout_item, default_value);
+          const Glib::ustring calculation = field->get_calculation();
+          const type_map_fields field_values = get_record_field_values_for_calculation(m_table_name, fieldPrimaryKey, primary_key_value);
+
+          //We need the connection when we run the script, so that the script may use it.
+          sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window());
+
+          const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field->get_glom_type(), calculation, field_values, document, m_table_name, sharedconnection->get_gda_connection());
+          set_entered_field_data(layout_item, value);
+        }
+
+        //Use default values (These are also specified in postgres as part of the field definition,
+        //so we could theoretically just not specify it here.
+        //TODO_Performance: Add has_default_value()?
+        if(Conversions::value_is_empty(value))
+        {
+          const Gnome::Gda::Value default_value = field->get_default_value();
+          if(!Conversions::value_is_empty(default_value))
+          {
+            set_entered_field_data(layout_item, default_value);
+          }
         }
       }
     }
@@ -245,45 +228,84 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
         Gnome::Gda::Value value;
 
         const sharedptr<const Field>& field = layout_item->get_full_field_details();
-
-        //Use the specified (generated) primary key value, if there is one:
-        if(primary_key_name == field_name && !GlomConversions::value_is_empty(primary_key_value))
+        if(field)
         {
-          value = primary_key_value;
-        }
-        else
-        {
-          if(use_entered_data || !field->get_calculation().empty()) //TODO_Performance: Use a get_has_calculation() method.
-            value = get_entered_field_data(layout_item);
-        }
-
-        Glib::ustring strFieldValue = field->sql(value);
-
-        if(!strFieldValue.empty())
-        {
-          if(!strNames.empty())
+          //Use the specified (generated) primary key value, if there is one:
+          if(primary_key_name == field_name && !Conversions::value_is_empty(primary_key_value))
           {
-            strNames += ", ";
-            strValues += ", ";
+            value = primary_key_value;
+          }
+          else
+          {
+            if(use_entered_data)
+              value = get_entered_field_data(layout_item);
           }
 
-          strNames += "\"" + field_name + "\"";
-          strValues += strFieldValue;
+          /* //TODO: This would be too many small queries when adding one record.
+          //Check whether the value meets uniqueness constraints:
+          if(field->get_primary_key() || field->get_unique_key())
+          {
+            if(!get_field_value_is_unique(m_table_name, layout_item, value))
+            {
+              //Ignore this field value. TODO: Warn the user about it.
+            } 
+          }
+          */
 
-          map_added[field_name] = true;
+          Glib::ustring strFieldValue = field->sql(value);
+
+          if(!strFieldValue.empty())
+          {
+            if(!strNames.empty())
+            {
+              strNames += ", ";
+              strValues += ", ";
+            }
+  
+            strNames += "\"" + field_name + "\"";
+            strValues += strFieldValue;
+  
+            map_added[field_name] = true;
+          }
         }
       }
     }
   }
 
   //Put it all together to create the record with these field values:
+  Glib::RefPtr<Gnome::Gda::DataModel> result;
   if(!strNames.empty() && !strValues.empty())
   {
-    Glib::ustring strQuery = "INSERT INTO \"" + m_strTableName + "\" (" + strNames + ") VALUES (" + strValues + ")";
-    return Query_execute(strQuery);
+    const Glib::ustring strQuery = "INSERT INTO \"" + m_table_name + "\" (" + strNames + ") VALUES (" + strValues + ")";
+    result = query_execute(strQuery, get_app_window());
+    if(result)
+    {
+      set_primary_key_value(row, primary_key_value); //Needed by Box_Data_List::on_adddel_user_changed().
+
+      //Update any lookups, related fields, or calculations:
+      for(type_vecLayoutFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
+      {
+         sharedptr<const LayoutItem_Field> layout_item = *iter;
+         
+         //TODO_Performance: We just set this with set_entered_field_data() above. Maybe we could just remember it.
+         const Gnome::Gda::Value field_value = get_entered_field_data(layout_item);
+
+         LayoutFieldInRecord field_in_record(layout_item, m_table_name, fieldPrimaryKey, primary_key_value);
+
+         //Get-and-set values for lookup fields, if this field triggers those relationships:
+         do_lookups(field_in_record, row, field_value);
+
+         //Update related fields, if this field is used in the relationship:
+         refresh_related_fields(field_in_record, row, field_value);
+      }
+    }
+    else
+    {
+      std::cerr << "Box_Data::record_new(): INSERT returned null DataModel." << std::endl; 
+    }
   }
-  else
-    return Glib::RefPtr<Gnome::Gda::DataModel>();
+
+  return result; //Failed.
 }
 
 void Box_Data::set_unstored_data(bool bVal)
@@ -301,7 +323,7 @@ void Box_Data::create_layout()
   set_unstored_data(false);
 
   //Cache the table information, for performance:
-  m_TableFields = get_fields_for_table(m_strTableName);
+  m_TableFields = get_fields_for_table(m_table_name);
 }
 
 bool Box_Data::fill_from_database()
@@ -333,7 +355,7 @@ void Box_Data::show_layout_dialog()
 {
   if(m_pDialogLayout)
   {
-    m_pDialogLayout->set_document(m_layout_name, get_document(), m_strTableName, m_FieldsShown); //TODO: Use m_TableFields?
+    m_pDialogLayout->set_document(m_layout_name, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
     m_pDialogLayout->show();
   }
 }
@@ -349,12 +371,12 @@ void Box_Data::on_dialog_layout_hide()
 
 Box_Data::type_vecLayoutFields Box_Data::get_fields_to_show() const
 {
-  if(m_strTableName.empty())
+  if(m_table_name.empty())
   {
     return type_vecLayoutFields();
   }
   else
-    return get_table_fields_to_show(m_strTableName);
+    return get_table_fields_to_show(m_table_name);
 }
 
 Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::ustring& table_name) const
@@ -379,7 +401,7 @@ Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& ta
   //This is a workaround for postgres problems. Ideally, we need to use the postgres serial type and find out how to get the generated value after we add a row.
 
   double result = 0;
-  Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute("SELECT max(" + field_name + ") FROM \"" + table_name + "\"");
+  Glib::RefPtr<Gnome::Gda::DataModel> data_model = query_execute("SELECT max(" + field_name + ") FROM \"" + table_name + "\"");
   if(!data_model && data_model->get_n_rows())
     handle_error();
   else
@@ -391,26 +413,27 @@ Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& ta
     //if(value.is_number())
     //  result = value.get_integer();
     //else
-    result = util_decimal_from_string(value.to_string());
+    result = decimal_from_string(value.to_string());
 
     ++result; 
   }
 
   //Get a string representation of the number, so we can put it back in a NUMERIC Gda::Value:
 
-  return GlomConversions::parse_value(result);
+  return Conversions::parse_value(result);
   */
 }
 
 /** Get the shown fields that are in related tables, via a relationship using @a field_name changes.
  */
-Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring& field_name) const
+Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const sharedptr<const LayoutItem_Field>& field) const
 {
   type_vecLayoutFields result;
 
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
   if(document)
   {
+    const Glib::ustring field_name = field->get_name(); //At the moment, relationships can not be based on related fields on the from side.
     for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end();  ++iter)
     {
       sharedptr<LayoutItem_Field> layout_field = *iter;
@@ -418,7 +441,7 @@ Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring&
       if(layout_field->get_has_relationship_name())
       {
         //Get the relationship information:
-        sharedptr<const Relationship> relationship = document->get_relationship(m_strTableName, layout_field->get_relationship_name());
+        sharedptr<const Relationship> relationship = document->get_relationship(m_table_name, layout_field->get_relationship_name());
         if(relationship)
         {
           //If the relationship uses the specified field:
@@ -435,282 +458,53 @@ Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring&
   return result;
 }
 
-Box_Data::type_field_calcs Box_Data::get_calculated_fields(const Glib::ustring& field_name)
+void Box_Data::refresh_related_fields(const LayoutFieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& /* field_value */)
 {
-  type_field_calcs result;
+  if(field_in_record_changed.m_table_name != m_table_name)
+    return; //TODO: Handle these too?
 
-  const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
-  if(document)
+  //Get values for lookup fields, if this field triggers those relationships:
+  //TODO_performance: There is a LOT of iterating and copying here.
+  //const Glib::ustring strFieldName = field_in_record_changed.m_field->get_name();
+  type_vecLayoutFields fieldsToGet = get_related_fields(field_in_record_changed.m_field);
+
+  if(!fieldsToGet.empty())
   {
-    //Examine all fields, not just the the shown fields.
-    for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end();  ++iter)
-    {
-      sharedptr<const Field> field = *iter;
+    const Glib::ustring query = Utils::build_sql_select_with_key(field_in_record_changed.m_table_name, fieldsToGet, field_in_record_changed.m_key, field_in_record_changed.m_key_value);
 
-      //Does this field's calculation use the field?
-      const Field::type_list_strings fields_triggered = field->get_calculation_fields();
-      Field::type_list_strings::const_iterator iterFind = std::find(fields_triggered.begin(), fields_triggered.end(), field_name);
-      if(iterFind != fields_triggered.end())
-      {
-        CalcInProgress item;
-        item.m_field = field;
-
-        result[field->get_name()] = item;
-      }
-    }
-  }
-
-  return result;
-}
-
-/** Get the fields whose values should be looked up when @a field_name changes, with
- * the relationship used to lookup the value.
- */
-Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& field_name) const
-{
-  type_list_lookups result;
-
-  const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
-  if(document)
-  {
-    //Examine all fields, not just the the shown fields (m_Fields):
-    for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end();  ++iter)
-    {
-      sharedptr<Field> field = *iter;
-
-      //Examine each field that looks up its data from a relationship:
-      if(field->get_is_lookup())
-      {
-        //Get the relationship information:
-        sharedptr<Relationship> relationship = field->get_lookup_relationship();
-        if(relationship)
-        {
-          //If the relationship is triggererd by the specified field:
-          if(relationship->get_from_field() == field_name)
-          {
-            //Add it:
-            sharedptr<LayoutItem_Field> item = sharedptr<LayoutItem_Field>::create();
-            item->set_full_field_details(field);
-            result.push_back( type_pairFieldTrigger(item, relationship) );
-          }
-        }
-      }
-    }
-  }
-
-  return result;
-}
-
-Gnome::Gda::Value Box_Data::get_lookup_value(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& source_field, const Gnome::Gda::Value& key_value)
-{
-  Gnome::Gda::Value result;
-
-  sharedptr<Field> to_key_field = get_fields_for_table_one_field(relationship->get_to_table(), relationship->get_to_field());
-  if(to_key_field)
-  {
-    Glib::ustring strQuery = "SELECT \"" + relationship->get_to_table() + "\".\"" + source_field->get_name() + "\" FROM \"" +  relationship->get_to_table() + "\"";
-    strQuery += " WHERE \"" + relationship->get_to_field() + "\" = " + to_key_field->sql(key_value);
-
-    Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(strQuery);
-    if(data_model && data_model->get_n_rows())
-    {
-      //There should be only 1 row. Well, there could be more but we will ignore them.
-      result = data_model->get_value_at(0, 0);
-    }
-    else
+    Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute(query, get_app_window());
+    if(!result)
     {
+      g_warning("Box_Data_List::refresh_related_fields(): no result.");
       handle_error();
     }
-  }
-
-  return result;
-}
-
-void Box_Data::calculate_field(const sharedptr<const Field>& field, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value)
-{
-  const Glib::ustring field_name = field->get_name();
-  //g_warning("Box_Data::calculate_field(): field_name=%s", field_name.c_str());
-
-  //Do we already have this in our list?
-  type_field_calcs::iterator iterFind = m_FieldsCalculationInProgress.find(field_name);
-  if(iterFind == m_FieldsCalculationInProgress.end()) //If it was not found.
-  {
-    //Add it:
-    CalcInProgress item;
-    item.m_field = field;
-    m_FieldsCalculationInProgress[field_name] = item;
-
-    iterFind = m_FieldsCalculationInProgress.find(field_name); //Always succeeds.
-  }
-
-  CalcInProgress& refCalcProgress = iterFind->second;
-
-  //Use the previously-calculated value if possible:
-  if(refCalcProgress.m_calc_in_progress)
-  {
-    //g_warning("  Box_Data::calculate_field(): Circular calculation detected. field_name=%s", field_name.c_str());
-    //refCalcProgress.m_value = GlomConversions::get_empty_value(field->get_glom_type()); //Give up.
-  }
-  else if(refCalcProgress.m_calc_finished)
-  {
-    //g_warning("  Box_Data::calculate_field(): Already calculated.");
-
-    //Don't bother calculating it again. The correct value is already in the database and layout.
-  }
-  else
-  {
-    //g_warning("  Box_Data::calculate_field(): setting calc_in_progress: field_name=%s", field_name.c_str());
-
-    refCalcProgress.m_calc_in_progress = true; //Let the recursive calls to calculate_field() check this.
-
-    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-    layout_item->set_full_field_details(refCalcProgress.m_field);
-
-    //Calculate dependencies first:
-    const Field::type_list_strings fields_needed = field->get_calculation_fields();
-    for(Field::type_list_strings::const_iterator iterNeeded = fields_needed.begin(); iterNeeded != fields_needed.end(); ++iterNeeded)
+    else
     {
-      sharedptr<const Field> field_needed = get_document()->get_field(m_strTableName, *iterNeeded);
-      if(field_needed)
+      //Field contents:
+      if(result->get_n_rows())
       {
-        if(field_needed->get_has_calculation())
+        type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
+
+        guint cols_count = result->get_n_columns();
+        for(guint uiCol = 0; uiCol < cols_count; uiCol++)
         {
-          //g_warning("  calling calculate_field() for %s", iterNeeded->c_str());
-          calculate_field(field_needed, primary_key, primary_key_value);
-        }
-        else
-        {
-          //g_warning("  not a calculated field->");
+          const Gnome::Gda::Value value = result->get_value_at(uiCol, 0 /* row */);
+          sharedptr<LayoutItem_Field> layout_item = *iterFields;
+
+          //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
+          //g_warning("  value_as_string=%s", value.to_string().c_str());
+
+          //m_AddDel.set_value(row, layout_item, value);
+          set_entered_field_data(row, layout_item, value);
+          //g_warning("addedel size=%d", m_AddDel.get_count());
+
+          ++iterFields;
         }
       }
-    }
-
-
-    //m_FieldsCalculationInProgress has changed, probably invalidating our iter, so get it again:
-    iterFind = m_FieldsCalculationInProgress.find(field_name); //Always succeeds.
-    CalcInProgress& refCalcProgress = iterFind->second;
-
-    //Check again, because the value miight have been calculated during the dependencies.
-    if(refCalcProgress.m_calc_finished)
-    {
-      //We recently calculated this value, and set it in the database and layout, so don't waste time doing it again:
-    }
-    else
-    {
-      //recalculate:
-      //TODO_Performance: We don't know what fields the python calculation will use, so we give it all of them:
-      const type_map_fields field_values = get_record_field_values(primary_key_value);
-
-      sharedptr<const Field> field = refCalcProgress.m_field;
-      refCalcProgress.m_value  = glom_evaluate_python_function_implementation(field->get_glom_type(), field->get_calculation(), field_values,
-            get_document(), m_strTableName);
-
-      refCalcProgress.m_calc_finished = true;
-      refCalcProgress.m_calc_in_progress = false;
-
-      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-      layout_item->set_full_field_details(field);
-
-      //show it:
-      set_entered_field_data(layout_item, refCalcProgress.m_value );
-
-      //Add it to the database (even if it is not shown in the view)
-      //Using true for the last parameter means we use existing calculations where possible,
-      //instead of recalculating a field that is being calculated already, and for which this dependent field is being calculated anyway.
-      set_field_value_in_database(layout_item, refCalcProgress.m_value, primary_key, primary_key_value, true); //This triggers other recalculations/lookups.
+      else
+       g_warning("Box_Data_List::refresh_related_fields(): no records found.");
     }
   }
-
-}
-
-
-void Box_Data::do_calculations(const sharedptr<const LayoutItem_Field>& field_changed, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value, bool first_calc_field)
-{
-  //g_warning("Box_Data::do_calculations(): triggered by =%s", field_changed->get_name().c_str());
-
-  if(first_calc_field)
-  {
-    //g_warning("  clearing m_FieldsCalculationInProgress");
-    m_FieldsCalculationInProgress.clear();
-  }
-
-  //Recalculate fields that are triggered by a change of this field's value, not including calculations that these calculations use.
-  type_field_calcs calculated_fields = get_calculated_fields(field_changed->get_name()); //TODO: Just return the Fields, not the CalcInProgress.
-  for(type_field_calcs::iterator iter = calculated_fields.begin(); iter != calculated_fields.end(); ++iter)
-  {
-    sharedptr<const Field> field = iter->second.m_field;
-
-    calculate_field(field, primary_key, primary_key_value); //And any dependencies.
-
-    //Calculate anything that depends on this.
-    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-    layout_item->set_full_field_details(field);
-
-    do_calculations(layout_item, primary_key, primary_key_value, false /* recurse, reusing m_FieldsCalculationInProgress */);
-  }
-
-  if(first_calc_field)
-    m_FieldsCalculationInProgress.clear();
-}
-
-bool Box_Data::set_field_value_in_database(const sharedptr<const LayoutItem_Field>& field_layout, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
-{
-  return set_field_value_in_database(Gtk::TreeModel::iterator(), field_layout, field_value, primary_key_field, primary_key_value, use_current_calculations);
-}
-
-bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_layout, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
-{
-  //row is invalid, and ignored, for Box_Data_Details.
-
-  const sharedptr<const Field> field = field_layout->get_full_field_details();
-
-  const Glib::ustring field_name = field_layout->get_name();
-  if(!field_name.empty()) //This should not happen.
-  {
-    Glib::ustring table_name;
-    if(field_layout->get_has_relationship_name())
-    {
-      //The field is in a related table.
-      sharedptr<Relationship> rel = field_layout->get_relationship();
-      if(rel)
-        table_name = rel->get_to_table();
-    }
-    else
-      table_name = m_strTableName;
-
-    if(table_name.empty())
-    {
-      g_warning("Box_Data::set_field_value_in_database(): table_name is empty.");
-      return false;
-    }
-
-    Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
-    strQuery += " SET \"" + field_name + "\" = " + field->sql(field_value);
-    strQuery += " WHERE \"" + primary_key_field->get_name() + "\" = " + primary_key_field->sql(primary_key_value);
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(strQuery);  //TODO: Handle errors
-    if(!datamodel)
-    {
-      g_warning("Box_Data::set_field_value_in_database(): UPDATE failed.");
-      return false; //failed.
-    }
-
-    //Get-and-set values for lookup fields, if this field triggers those relationships:
-    do_lookups(row, field_layout, field_value, primary_key_field, primary_key_value);
-
-    //Update related fields, if this field is used in the relationship:
-    refresh_related_fields(row, field_layout, field_value, primary_key_field, primary_key_value);
-
-    //Calculate any dependent fields.
-    //TODO: Make lookups part of this?
-    //Prevent circular calculations during the recursive do_calculations:
-    {
-      //Recalculate any calculated fields that depend on this calculated field.
-      //g_warning("Box_Data::set_field_value_in_database(): calling do_calculations");
-      do_calculations(field_layout, primary_key_field, primary_key_value, !use_current_calculations);
-    }
-  }
-
-  return true;
 }
 
 Document_Glom::type_mapLayoutGroupSequence Box_Data::get_data_layout_groups(const Glib::ustring& layout)
@@ -720,17 +514,20 @@ Document_Glom::type_mapLayoutGroupSequence Box_Data::get_data_layout_groups(cons
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
   if(document)
   {
-    if(!m_strTableName.empty())
+    if(!m_table_name.empty())
     {
       //Get the layout information from the document:
-      layout_groups = document->get_data_layout_groups_plus_new_fields(layout, m_strTableName);
+      layout_groups = document->get_data_layout_groups_plus_new_fields(layout, m_table_name);
 
-      const Privileges table_privs = get_current_privs(m_strTableName);
+      const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
       //Fill in the field information for the fields mentioned in the layout:
       for(Document_Glom::type_mapLayoutGroupSequence::iterator iterGroups = layout_groups.begin(); iterGroups != layout_groups.end(); ++iterGroups)
       {
         fill_layout_group_field_info(iterGroups->second, table_privs);
+
+        //std::cout << "debug: Box_Data::get_data_layout_groups: " << std::endl;
+        //iterGroups->second->debug();
       }
     }
   }
@@ -757,7 +554,7 @@ void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group,
       {
         //Get the full field information:
         const Glib::ustring relationship_name = item_field->get_relationship_name();
-        sharedptr<const Relationship> relationship = document->get_relationship(m_strTableName, relationship_name);
+        sharedptr<const Relationship> relationship = document->get_relationship(m_table_name, relationship_name);
         if(relationship)
         {
           sharedptr<Field> field = get_fields_for_table_one_field(relationship->get_to_table(), item->get_name());
@@ -766,7 +563,7 @@ void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group,
             item_field->set_full_field_details(field);
 
             //TODO_Performance: Don't do this repeatedly for the same table.
-            const Privileges privs = get_current_privs(relationship->get_to_table());
+            const Privileges privs = Privs::get_current_privs(relationship->get_to_table());
             item_field->m_priv_view = privs.m_view;
             item_field->m_priv_edit = privs.m_edit;
           }
@@ -775,7 +572,7 @@ void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group,
       else
       {
         //Get the field info:
-        sharedptr<Field> field = get_fields_for_table_one_field(m_strTableName, item_field->get_name());
+        sharedptr<Field> field = get_fields_for_table_one_field(m_table_name, item_field->get_name());
         if(field)
         {
           item_field->set_full_field_details(field); //TODO_Performance: Just use this as the output arg?
@@ -800,9 +597,9 @@ bool Box_Data::record_delete(const Gnome::Gda::Value& primary_key_value)
 {
 
   sharedptr<Field> field_primary_key = get_field_primary_key();
-  if(field_primary_key && !GlomConversions::value_is_empty(primary_key_value))
+  if(field_primary_key && !Conversions::value_is_empty(primary_key_value))
   {
-    return Query_execute( "DELETE FROM \"" + m_strTableName + "\" WHERE \"" + m_strTableName + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->sql(primary_key_value) );
+    return query_execute( "DELETE FROM \"" + m_table_name + "\" WHERE \"" + m_table_name + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->sql(primary_key_value), get_app_window());
   }
   else
   {
@@ -827,17 +624,6 @@ g_warning("debug: Box_Data::get_field()");
 }
 */
 
-Glib::ustring Box_Data::build_sql_select(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value)
-{
-  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a record to show:
-  {
-    const Glib::ustring where_clause = "\"" + table_name + "\".\"" + primary_key_field->get_name() + "\" = " + primary_key_field->sql(primary_key_value);
-    return GlomUtils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
-  }
-
-  return Glib::ustring();
-}
-
 /*
 Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause)
 {
@@ -909,11 +695,11 @@ bool Box_Data::get_related_record_exists(const sharedptr<const Relationship>& re
 
   //TODO_Performance: It's very possible that this is slow.
   //We don't care how many records there are, only whether there are more than zero.
-  const Glib::ustring to_field = relationship->get_to_table() = relationship->get_to_field();
+  const Glib::ustring to_field = relationship->get_to_field();
 
   //TODO_Performance: Is this the best way to just find out whether there is one record that meets this criteria?
   const Glib::ustring query = "SELECT \"" + to_field + "\" FROM \"" + relationship->get_to_table() + "\" WHERE \"" + to_field + "\" = " + key_field->sql(key_value);
-  Glib::RefPtr<Gnome::Gda::DataModel> records = Query_execute(query);
+  Glib::RefPtr<Gnome::Gda::DataModel> records = query_execute(query, get_app_window());
   if(!records)
     handle_error();
   else
@@ -957,7 +743,7 @@ bool Box_Data::add_related_record_for_field(const sharedptr<const LayoutItem_Fie
       const bool key_is_auto_increment = primary_key_field->get_auto_increment();
 
       //If we would have to set an otherwise auto-increment key to add the record.
-      if( key_is_auto_increment && !GlomConversions::value_is_empty(primary_key_value) )
+      if( key_is_auto_increment && !Conversions::value_is_empty(primary_key_value) )
       {
         //Warn the user:
         //TODO: Make the field insensitive until it can receive data, so people never see this dialog.
@@ -985,7 +771,7 @@ bool Box_Data::add_related_record_for_field(const sharedptr<const LayoutItem_Fie
         }
 
         const Glib::ustring strQuery = "INSERT INTO \"" + relationship->get_to_table() + "\" (\"" + primary_key_field->get_name() + "\") VALUES (" + primary_key_field->sql(primary_key_value) + ")";
-        bool test = Query_execute(strQuery);
+        bool test = query_execute(strQuery, get_app_window());
         if(test)
         {
           if(key_is_auto_increment)
@@ -1019,7 +805,7 @@ bool Box_Data::add_related_record_for_field(const sharedptr<const LayoutItem_Fie
                 {
                   const Glib::ustring strQuery = "UPDATE \"" + relationship->get_from_table() + "\" SET \"" + relationship->get_from_field() + "\" = " + primary_key_field->sql(primary_key_value) +
                     " WHERE \"" + relationship->get_from_table() + "\".\"" + parent_primary_key_field->get_name() + "\" = " + parent_primary_key_field->sql(parent_primary_key_value);
-                  const bool test = Query_execute(strQuery);
+                  const bool test = query_execute(strQuery, get_app_window());
                   return test;
                 }
               }
@@ -1059,3 +845,6 @@ bool Box_Data::confirm_delete_record()
   const int response = dialog.run();
   return (response == Gtk::RESPONSE_OK);
 }
+
+} //namespace Glom
+
