@@ -26,6 +26,9 @@
 #include <glom/libglom/utils.h>
 
 
+namespace Glom
+{
+
 DbTreeModelRow::DbTreeModelRow()
 : m_values_retrieved(false),
   m_removed(false),
@@ -207,7 +210,7 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
 
   if(m_connection && !m_found_set.m_table_name.empty() && m_get_records)
   {
-    const Glib::ustring sql_query = GlomUtils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_sort_clause);
+    const Glib::ustring sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_sort_clause);
 
     //std::cout << "DbTreeModel: Executing SQL: " << sql_query << std::endl << std::endl;
     try
@@ -224,6 +227,7 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
       m_data_model_rows_count = 0;
       m_data_model_columns_count = m_columns_count;
 
+      std::cerr << "DbTreeModel::refresh_from_database(): error executing SQL: " << sql_query << std::endl;
       ConnectionPool::handle_error();
       return false; //No records were found.
     }
@@ -891,3 +895,5 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
     found = 0;
   }
 }
+
+} //namespace Glom
